clear all

addpath('../gpu_warmup');
addpath('../images');

%
% clean image
%
%I   =  imread('shape.jpg');       % 128x128
%I   =  imread('Cameraman.bmp');   % 256x256
I   =  imread('Barbara.png');     % 512x512
%I   =  imread('man.tiff');      % 1024x1024
%I   =  imread('earth2k.jpg');      % 2048x2048

%
% parameters
%
solve_GPU = 1;
solve_host = 1;
plotresults = 0;

lbd =  0.041;
gap_tol_vals = [1.e-2 1.e-4 1.e-6];

n_runs = length(gap_tol_vals);

time_host = zeros(n_runs,1);
time_gpu  = zeros(n_runs,1);
iter_host = zeros(n_runs,1);
iter_gpu  = zeros(n_runs,1);
mse_host  = zeros(n_runs,1);
mse_gpu   = zeros(n_runs,1);

nit = 1000;    %max number of iterations
noise_var = 0.01;

n = length(I); 

f = double(imnoise(I,'gaussian',0,noise_var)); 

fprintf('\nDenoising: image size %dx%d with gaussian noise mean %.2f std %.4f\n', n, n, 0, sqrt(noise_var));

if solve_GPU
    fprintf('\nSolving on GPU,  lbd=%g\n', lbd);
    % warming-up GPU
    gpu_warmup(2);
    i=1;
    for gap_tol = gap_tol_vals,
	[u_gpu, w1,w2, Energy,Dgap_gpu, iter_gpu(i), total_time, exec_time] = ...
	    TV_PDHG_gpu(single(zeros(n)),single(zeros(n)),single(f),lbd,nit,gap_tol);
	time_gpu(i) = total_time;
	singleI = single(I);
	mse_gpu(i) = sum(((u_gpu(:)-singleI(:)) ./ 256).^2) / length(I(:));
	%snr_gpu(i) = snr(u_gpu, singleI);
	fprintf(' gap_tol=%8g: iter=%4d, time=%8.4f, mse=%8.2e\n', gap_tol, iter_gpu(i), time_gpu(i), mse_gpu(i));
	i = i+1;
    end
end

if solve_host
    fprintf('\nSolving on host, lbd=%g\n', lbd);
    i=1;
    for gap_tol = gap_tol_vals,
	[u_host, w1,w2, Energy,Dgap_host, time_host(i), iter_host(i)] = ...
	    TV_PDHG_host(zeros(n),zeros(n),f,lbd,nit,gap_tol);
	doubleI = double(I);
	mse_host(i) = sum(((u_host(:)-doubleI(:))./ 256).^2) / length(I(:));
	%snr_host(i) = snr(u_host, doubleI);
	fprintf(' gal_tol=%8g: iter=%4d, time=%8.4f, mse=%8.2e\n', gap_tol, iter_host(i), time_host(i), mse_host(i));
	i = i+1;
    end
end

if solve_host && solve_GPU
  fprintf('\n\nLatex table....\n\n');
  fprintf('\\begin{table}\\centering\n');
  fprintf('\\begin{tabular}{|c||cc|cc|cc|}\\hline\n');
  fprintf('\\multirow{2}{*}{Gap tol} & \\multicolumn{2}{|c|}{CPU} & \\multicolumn{2}{|c|}{GPU} & \\multicolumn{2}{|c|}{Speedup}\\\\\n');
  fprintf('\\cline{2-7}\n')
  fprintf(' & iters & time (s) & iters & time (s) & total & iter\\\\\n');
  fprintf('\\hline\n');
  i = 1;
  for gap_tol = gap_tol_vals,
    fprintf('%f & %d & %.2f & & %d & %.2f & %.0f & %.0f\\\\\n', gap_tol, iter_host(i), time_host(i), iter_gpu(i), time_gpu(i), time_host(i)/time_gpu(i), (time_host(i)*iter_gpu(i))/(time_gpu(i)*iter_host(i))); 
    i = i+1;
  end
  fprintf('\\hline\n');
  fprintf('\\end{tabular}\n');
  fprintf('\\caption{Computational results for denoising for image dimension $%d\\times %d$ ($\\lambda$=%.3f)\\label{ta:image.%d}}\n', n, n, lbd, n)
  fprintf('\\end{table}\n\n');
%
end

% only shows the last image
if plotresults
    figure;
    subplot(221);
    image(I);
    title('original image');
    subplot(222)
    image(f);
    title('noisy image');
    subplot(223);
    if solve_host
	image(u_host);
    end
    title('recovered image(CPU)');
    subplot(224);
    if solve_GPU
	image(u_gpu);
    end
    title('recovered image(GPU)');

    colormap(gray(256));
end

